package com.alibaba.alink.opensdksample.net;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.text.TextUtils;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.alibaba.alink.opensdksample.R;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.alink.business.alink.ALinkBusinessEx;
import com.aliyun.alink.business.mtop.MTopResponse;
import com.aliyun.alink.business.mtop.MTopResponseData;
import com.aliyun.alink.sdk.net.anet.api.AError;
import com.aliyun.alink.sdk.net.anet.api.transitorynet.TransitoryRequest;
import com.aliyun.alink.sdk.net.anet.api.transitorynet.TransitoryResponse;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;

public class AlinkBusinessExDemoActivity extends Activity {

    EditText editTextMethodName, editTextMethodArgs;
    Button buttonGo;
    TextView textViewResponse;


    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_alink_business_ex_demo);

        editTextMethodName = (EditText) findViewById(R.id.edittext_method_name);
        editTextMethodArgs = (EditText) findViewById(R.id.edittext_method_args);
        buttonGo = (Button) findViewById(R.id.button_go);
        textViewResponse = (TextView) findViewById(R.id.textview_response);
    }

    @Override
    protected void onPostCreate(@Nullable Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);

        buttonGo.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                String method = editTextMethodName.getText().toString();
                String args = editTextMethodArgs.getText().toString();

                JSONObject jsonArgs = null;
                HashMap<String, Object> mapArgs = null;

                if (TextUtils.isEmpty(method)) {
                    Toast.makeText(AlinkBusinessExDemoActivity.this,
                            "方法名无效",
                            Toast.LENGTH_LONG).show();
                    return;
                }

                if (!TextUtils.isEmpty(args)) {

                    try {
                        jsonArgs = JSON.parseObject(args);
                    } catch (Exception e) {
                        e.printStackTrace();
                        Toast.makeText(AlinkBusinessExDemoActivity.this,
                                "参数无效（非法的JSON格式）",
                                Toast.LENGTH_LONG).show();
                        return;
                    }

                }

                if (null != jsonArgs) {

                    mapArgs = new HashMap<>(jsonArgs.size());

                    Iterator<Map.Entry<String, Object>> iterator = jsonArgs.entrySet().iterator();
                    while (iterator.hasNext()) {
                        Map.Entry<String, Object> entry = iterator.next();
                        mapArgs.put(entry.getKey(), entry.getValue());
                    }

                }

                doRequest(method, mapArgs);

            }
        });

        if (null == savedInstanceState
                && null != getIntent()) {

            Intent intent = getIntent();
            String methodName = intent.getStringExtra("mn");
            String methodArgs = intent.getStringExtra("ma");

            if (TextUtils.isEmpty(methodName)) {
                methodName = "mtop.openalink.app.core.alinktime.get";
            }

            editTextMethodName.setText(methodName);
            editTextMethodArgs.setText(methodArgs);

        }
    }

    void doRequest(String method, HashMap<String, Object> args) {

        TransitoryRequest transitoryRequest = new TransitoryRequest();
        transitoryRequest.setMethod(method);

        if (null != args) {
            transitoryRequest.setParams(args);
        }

        ALinkBusinessEx biz = new ALinkBusinessEx();
        biz.request(transitoryRequest, new ALinkBusinessEx.IListener() {
            @Override
            public void onSuccess(TransitoryRequest transitoryRequest, TransitoryResponse transitoryResponse) {
                MTopResponse response = (MTopResponse) transitoryResponse.getOriginResponseObject();
                MTopResponseData data = response.data;

                String detailMessage = "请求已返回";

                if ("1000".equalsIgnoreCase(data.code)) {
                    try {
                        textViewResponse.setText(JSON.toJSONString(data.data));
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                } else {
                    detailMessage = String.format(Locale.ENGLISH, "请求失败: code:%d,message:%s", data.code, data.description);
                    Toast.makeText(AlinkBusinessExDemoActivity.this, detailMessage, Toast.LENGTH_SHORT).show();
                }

                Toast.makeText(AlinkBusinessExDemoActivity.this, detailMessage, Toast.LENGTH_SHORT).show();
            }

            @Override
            public void onFailed(TransitoryRequest transitoryRequest, AError aError) {
                String detailMessage = String.format(Locale.ENGLISH, "请求失败: code:%d,message:%s", aError.getCode(), aError.getMsg());
                Toast.makeText(AlinkBusinessExDemoActivity.this, detailMessage, Toast.LENGTH_SHORT).show();
            }
        });
    }
}
